/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Author: PU20 Department. 
//  Tel: 03-5770066#8043
//  E-Mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: ADC Function 
//  ADC Channel 0 as ADC input
//  UART show ADC output result on hyper-terminal
//***********************************************************************************************************
#include"N79E825.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include <intrins.h>
#include <stdio.h>
#include <string.h>

void ADC_ISR() interrupt 11
{
    ADCCON &= CLR_BIT4;           //ADCI=0
    ADCCON &= CLR_BIT3;           //ADCS=0
}
//-----------------------------------------------------------------------------------------------------------
void EnableADC(UINT8 u8On)
{
    if (u8On==0)
        AUXR1 &= CLR_BIT2;        //disable
    else                      
        AUXR1 |= SET_BIT2;        //enable
}
//-----------------------------------------------------------------------------------------------------------
void EnableADC_INT()
{
    EADC = 1;
    EA = 1;
}
//-----------------------------------------------------------------------------------------------------------
void ADCchannel(UINT8 temp)
{
    ADCCON = ADCCON  & CLR_BIT1;       //0  0   0   ADC0 (P0.0)
    ADCCON = ADCCON  & CLR_BIT0;
    switch (temp)
    {
        case  0:
        break;
        case  1:
            ADCCON = ADCCON | SET_BIT0;//0  0   1   ADC0 (P0.1)
        break;
        case  2:
            ADCCON = ADCCON | SET_BIT1;//0  1   0   ADC0 (P0.2)
        break;
        case  3:
            ADCCON = ADCCON | SET_BIT1;
            ADCCON = ADCCON | SET_BIT0;//0  1   1   ADC0 (P0.3)
        break;
    }
}
//-----------------------------------------------------------------------------------------------------------
void TriggerADC(void)
{
    ADCCON &= CLR_BIT4;   //ADCI=0

    ADCCON |= SET_BIT3;   //ADCS=1

    PCON |= SET_BIT0;     //enter idle mode
}
//-----------------------------------------------------------------------------------------------------------
void main()
{
    InitialUART0_Timer1(9600);                  /* 9600 Baud Rate @ 11.0592MHz */
    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E825 Series ADC Demo Code.");
    printf ("\n*===================================================================");  
    printf ("\nADC Demo Start.\n");
  
    P0M2 = 0x00;          
    P0M1 = 0x78;          //P0.6~0.3 input only
                        
    P0ID = 0x78;          //disable digital P0.6~0.3    
    ADCchannel(0);    
    EnableADC(1);
    EnableADC_INT();
    
    while(1)
    {
        UINT16 u16ADCL;
        UINT16 u16ADC;

        TriggerADC();

        u16ADCL = ADCCON;
        u16ADCL = u16ADCL >> 6;                     //ADC[1:0]
            
        u16ADC = ADCH;                              
        u16ADC = ( u16ADC << 2 ) + u16ADCL | 0x0003;//ADC[9:2] + ADC[1:0]
        
        printf ("\nADC = 0x%d",u16ADC);//Show 10 bit ADC

        Delay1ms(1);   
    }
    while(1);
}








