/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Author: PU20 Department. 
//  Tel: 03-5770066#8043
//  E-Mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************

//***********************************************************************************************************
//  N79E82x-series I2C demo code, the Slave address = 0xA4
//
//   ____________            _____________ 
//  |            |   SDA    |             |
//  |            |<-------->|             |
//  |            |          |             |
//  | N79E82x(M) |          | N79E82x(S)  |
//  |(I2C_Master)|          | (I2C_Slave) |
//  |            |   SCL    |             |
//  |            |--------->|             |
//  |____________|          |_____________|
//
//  The protocol of I2C is same the "24LC64"
//***********************************************************************************************************

#include <stdio.h>
#include "N79E825.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"

#define SCL             P12
#define SDA             P13

#define I2C_CLOCK               13

//#define EEPROM_SLA              0xA0  //for I2C EEPROM address
#define EEPROM_SLA              0xA4  //for I2C Slave MCU address
#define EEPROM_WR               0
#define EEPROM_RD               1
#define PAGE_SIZE               32
#define ERROR_CODE              0x78

void Init_I2C(void);
void I2C_Error(void);
void I2C_Process(UINT8 u8DAT);
//========================================================================================================
void Init_I2C(void)
{
    /* Set I2C clock rate */
    I2CLK = I2C_CLOCK; 

    /* Enable I2C */
    ENS1 = 1;;                                   
}
//========================================================================================================
void I2C_Error(void)
{
    P0 = I2STATUS;
    P1 = ERROR_CODE;
    while (1);    
}
//========================================================================================================
void I2C_Process(UINT8 u8DAT)
{
    UINT32 u32Count;

//--------------------------------------------------------------------------------------------
//----  Page Write----------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

    STA=1;
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x08)                       //Check status value after every step
        I2C_Error();
    
    STA=0;                                      //STA=0
    I2DAT = EEPROM_SLA | EEPROM_WR;
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x18)              
        I2C_Error();

    I2DAT = 0x00;                               //address high for I2C EEPROM
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x28)              
        I2C_Error();
                    
    I2DAT = 0x00;                               //address low for I2C EEPROM
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x28)              
        I2C_Error();

    for (u32Count = 0; u32Count < PAGE_SIZE; u32Count++)
    {
        I2DAT = u8DAT;
        SI=0;
        while ((I2CON&0x08)==0x00);             //Check SI set or not
        if (I2STATUS != 0x28)              
            I2C_Error();

        u8DAT = ~u8DAT;        
    }

//--------------------------------------------------------------------------------------------
//----  Waitting the ready for I2C write------------------------------------------------------
//--------------------------------------------------------------------------------------------
    do{
        STO=1;
        SI=0;
        while ((I2CON&0x10)==0x10);        
        
        STA=1;                                  //Check if no ACK is returned by EEPROM, it is under timed-write cycle
        SI=0;
        while ((I2CON&0x08)==0x00);             //Check SI set or not
        if (I2STATUS != 0x08)                   //Check status value after every step
            I2C_Error();

        STA=0;
        I2DAT = EEPROM_SLA | EEPROM_WR;
        SI=0;
        while ((I2CON&0x08)==0x00);             //Check SI set or not
    }while (I2STATUS != 0x18);

    STO=1;
    SI=0;
    while ((I2CON&0x10)==0x10); 
//--------------------------------------------------------------------------------------------
//----  Page Read ----------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
    STA=1;
    SI=0;          
    while ((I2CON&0x08)==0x00);                 //Check SI set or not                                //Check SI set or not
    if (I2STATUS != 0x08)                       //Check status value after every step
        I2C_Error();

    STA=0;
    I2DAT = EEPROM_SLA | EEPROM_WR;
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x18)              
        I2C_Error();

    I2DAT = 0x00;                               //address high for I2C EEPROM
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x28)              
        I2C_Error();

    I2DAT = 0x00;                               //address low for I2C EEPROM
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x28)              
        I2C_Error();

    /* Repeated START */
    STA=1;                       
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x10)                       //Check status value after every step
        I2C_Error();
    
    STA=0;                                      //STA needs to be cleared after START codition is generated
    I2DAT = EEPROM_SLA | EEPROM_RD;
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x40)              
        I2C_Error();
    
    for (u32Count = 0; u32Count <PAGE_SIZE; u32Count++)
    {
        AA=1;
        SI=0;        
        while ((I2CON&0x08)==0x00);             //Check SI set or not

        if (I2STATUS != 0x50)              
            I2C_Error();
        
        if (I2DAT != u8DAT)             
            I2C_Error();
        u8DAT = ~u8DAT; 
    } 
    
    AA=0;
    SI=0;
    while ((I2CON&0x08)==0x00);                 //Check SI set or not
    if (I2STATUS != 0x58)              
        I2C_Error();

    STO=1;
    SI=0;
    while ((I2CON&0x10)==0x10);
}
//========================================================================================================
main(void)
{
    InitialUART0_Timer1(9600);                  /* 9600 Baud Rate @ 11.0592MHz */

    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E82x Series I2C Interrupt Demo Code. (N79E82x_M <--> N79E82x_S)");
    printf ("\n*===================================================================\n");

    printf ("\nPress Enter to start...");

    Init_I2C();                                 //initial I2C circuit
    
    I2C_Process(0x55);                          /* I2C Master will send 0x55,0xAA,.... to slave */
    
    P0=0x00;
    P1=0x00;
    while (1);
/* =================== */
}


//-----------------------------------------------------------------------------------------------------------
