/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Author: PU20 Department. 
//  Tel: 03-5770066#8043
//  E-Mail: MicroC-8bit@nuvoton.com
// ***********************************************************************************************************
//  Application:    Demo code for I2C salve interrupt
// ***********************************************************************************************************

#include <stdio.h>
#include "N79E825.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"

#define SCL     P12
#define SDA     P13

UINT8 data_received[34], data_num = 0;

void i2c_isr(void) interrupt 6
{
    switch (I2STATUS)
    {
    case 0x00:
        STO = 1;
        break;

    case 0x60:
        AA = 1;
        P0 = 0x60;
        break;

    case 0x80:
        P0 = 0x80;
        data_received[data_num] = I2DAT;
        data_num++;

        if (data_num == 34)
            AA = 0;
        else
            AA = 1;
        break;

    case 0x88:
        P0 = 0x88;
        data_received[data_num] = I2DAT;
        data_num = 0;
        AA = 1;
        break;

    case 0xA0:
        P0 = 0xA0;
        AA = 1;
        break;

    case 0xA8:
        P0 = 0xA0;
        I2DAT = data_received[data_num];
        data_num++;
        AA = 1;
        break;
    
    case 0xB8:
        P0 = 0xB8;
        I2DAT = data_received[data_num];
        data_num++;
        AA = 1;
        break;

    case 0xC0:
        P0 = 0xC0;
        AA = 1;
        break; 

    case 0xC8:
        P0 = 0xC8;
        AA = 1;
        break;        
    }

    SI = 0;
    while(STO);
}

/*****************************************************************************
*   Initialize I2C function     
*****************************************************************************/
void Init_I2C(void)
{
    SDA = 1;        //set SDA and SCL pins high
    SCL = 1;
    P2M1 |= 0x40;   //enable P1 Schmitt trigger input

    EI2 = 1;        //enable I2C interrupt by setting IE1 bit 0
    EA = 1;

    I2ADDR = 0xA4;  //define own slave address
    ENS1 = 1;       //enable I2C circuit
    AA = 1;
}

/*****************************************************************************
*   I2C slave mode main     
*****************************************************************************/
void main(void)
{
    InitialUART0_Timer1(9600);                  /* 9600 Baud Rate @ 11.0592MHz */
    Init_I2C();                                 // Initialize I2C function  
    while(1);
}





