/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Author: PU20 Department. 
//  Tel: 03-5770066#8043
//  E-Mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************
//  Application: Interrupt
//  Uart show 16 bit number to Hyper-terminal

//  the bit0=0 Show INT0 Interrupt once be enter. 
//  the bit0=1 Show INT0 Interrupt never be enter. 
//
//  the bit1=0 Show Timer0 Interrupt once be enter.   
//  the bit1=1 Show Timer0 Interrupt once be enter. 
//  The other bits indicate whether the other Interrupt be enter or not.
//***********************************************************************************************************

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//    <h> Interrupt  Enable
//                  <q0.0> INT0       
//                  <q0.1> Timer0   
//                  <q0.2> INT1     
//                  <q0.3> Timer1   
//                  <q0.4> Serial     
//                  <q0.5> BOD      
//                  <q0.6> I2C      
//                  <q0.7> KBI        
//                  <q0.8>  Comperator2         
//                  <q0.9>  Watch Dog           
//                  <q0.10> ADC                 
//                  <q0.11> Comperator1         
//                  <q0.12> PWM                 
//    </h>
//-------- <<< end of configuration section >>> ------------------------------
*/
#define EnableINT         0xFFFF
#include <stdio.h>
#include "N79E825.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"

UINT16 u16INT_Mointer;

void INT0_ISR() interrupt 0 
{
    u16INT_Mointer &= 0xfffe;
    IE0 = 0;
}
//-----------------------------------------------------------------------------------------------------------
void TIMER0_ISR() interrupt 1 
{
    u16INT_Mointer &= 0xfffd;
    TF0 = 0; 
}
//-----------------------------------------------------------------------------------------------------------
void INT1_ISR() interrupt 2 
{
    u16INT_Mointer &= 0xfffb;
    IE1 = 0;    
}
//-----------------------------------------------------------------------------------------------------------
void TIMER1_ISR() interrupt 3
{
    u16INT_Mointer &= 0xfff7;
    TF1 = 0;
}
//-----------------------------------------------------------------------------------------------------------
void UART_ISR() interrupt 4
{
    u16INT_Mointer &= 0xffef;
    RI = 0;
}
//-----------------------------------------------------------------------------------------------------------
void BOF_ISR() interrupt 5 
{
    u16INT_Mointer &= 0xffdf;
    PCON &= 0xDF;    
}
//-----------------------------------------------------------------------------------------------------------
void I2C_ISR() interrupt 6 
{
    u16INT_Mointer &= 0xffbf;
    I2CON &= 0xf7;
}
//-----------------------------------------------------------------------------------------------------------
void KBI_ISR() interrupt 7
{
    u16INT_Mointer &= 0xff7f;
    AUXR1 &= 0x7f;
}
//-----------------------------------------------------------------------------------------------------------
void CMF2_ISR() interrupt 8 
{
    u16INT_Mointer &= 0xfeff;
    CMP2 &= 0xfe;
}
//-----------------------------------------------------------------------------------------------------------
void WDT_ISR() interrupt 10
{
    BIT bitStatus;
    
    u16INT_Mointer &= 0xfdff;
    
    bitStatus=EA;
    EA   = 0;    
    TA   = 0xaa;
    TA   = 0x55;
    WDIF = 0; 
    EA   = bitStatus;
}
//-----------------------------------------------------------------------------------------------------------
void ADC_ISR() interrupt 11
{
    u16INT_Mointer &= 0xfbff;
    ADCCON &= 0xEF;  
}
//-----------------------------------------------------------------------------------------------------------
void CMF1_ISR() interrupt 12 
{
    u16INT_Mointer &= 0xf7ff;
    CMP1 &= 0xFE;    
}
//-----------------------------------------------------------------------------------------------------------
void PWM_ISR() interrupt 14 
{
    PWMCON3 &= 0xFE;
    u16INT_Mointer &= 0xefff;
}
//-----------------------------------------------------------------------------------------------------------
void main()
{    
    UINT16 EnableINT_Shadow=EnableINT;
    u16INT_Mointer = 0XFFFF;
    AUXR1 |= 0x04;              //enable ADC function    
                                //init_interrupts:
    IE     = 0x00;              //disable all interrupts
    EIE    = 0;                 //disable all interrupts
    IP     = 0;                 //set all interrupts Low priority
    IP0H   = 0;                 //set all interrupts Low priority
    IP1    = 0;                 //set all interrupts Low priority
    IP1H   = 0;                 //set all interrupts Low priority
    TCON   = 0;                 //clear TR0, TR1 and (IE0, TF0, IE1, TF1)
    SCON   = 0;                 //clear REN and (RI, TI)
    PCON   = 0;                 //clear BOF
    TA     = 0;                 //
    WDCON  = 0;                 //clear WDIF
    I2CON  = 0;                 //clear SI
    ADCCON = 0;                 //clear ADCI
    AUXR1  = 0;                 //clear KBF
    CMP1   = 0;                 //clear CMF1
    CMP2   = 0;                 //clear CMF2

    if(EnableINT_Shadow|0x0001) //INT0
    {
        IT0 = 1;                //int0 negative edge trigger
        IE0 = 1;                //set int0 interrupt flag
    }
    if (EnableINT_Shadow|0x0002)//Timer0
    {
        TF0 = 1;                //set timer0 interrupt flag       
    }
    if (EnableINT_Shadow|0x0004)//INT1
    {
        IT1 = 1;
        IE1 = 1;                //set int1 interrupt flag                
    }
    if (EnableINT_Shadow|0x0008)//Timer1
    {
        TF1 = 1;                //set timer1 interrupt flag         
    }
    if (EnableINT_Shadow|0x0010)//Serial
    {
        RI = 1;                 //set RI interrupt flag        
    }
    if (EnableINT_Shadow|0x0020)//BOD
    {  
        PCON |= 0x20;           //set BOF flag                   
    }
    if (EnableINT_Shadow|0x0040)//I2C
    {
        I2CON |= 0x08;          //set SI flag                 
    }
    if (EnableINT_Shadow|0x0080)//KBI
    {
        AUXR1 |= 0x80;          //set KBF flag               
    }
    if (EnableINT_Shadow|0x0100)//Cmp2
    {
        CMP2 |= 0x01;           //set CMF2 flag                 
    }
    if (EnableINT_Shadow|0x0200)//WDT
    {
        BIT bitStatus;
        bitStatus=EA;
        EA   = 0;      
        TA   = 0xaa;              
        TA   = 0x55;              
        WDIF = 1;               //set Watchdog interrupt flag        
        EA   = bitStatus;      
    }
    if (EnableINT_Shadow|0x0400)//ADC
    {
        ADCCON |= 0x10;         //set ADCI flag        
    }
    if (EnableINT_Shadow|0x0800)//Cmp1
    {
      CMP1 |= 0x01;             //set CMF1 flag               
    }   
    if (EnableINT_Shadow|0x1000)//PWM
    {
      PWMCON3 |= 0x01;          //set BKF flag         
    }

    EIE =0x3f;
    IE  =0x7f;                  //enable all interrupt
    EA  =1;

    Delay1ms(10);
    EA  = 0;                    //Disable all interrupt
    EIE = 0x0;
    IE  = 0x0;

    InitialUART0_Timer1(9600);  /* 9600 Baud Rate @ 11.0592MHz */
    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E825 Series Interrupt Demo Code.");
    printf ("\n*===================================================================");  
    printf ("\nInterrupt Demo Start.");
    printf ("\nthe bit0=0 Show INT0 Interrupt once be enter.");
    printf ("\nthe bit0=1 Show INT0 Interrupt never be enter.");
    printf ("\n...The other bits indicate whether the other Interrupt be enter or not.");

    printf ("\n INT Check  = 0x%2X",u16INT_Mointer);
    while(1);
}








