/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2011 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

//***********************************************************************************************************
//  Nuvoton Technoledge Corp. 
//  Author: PU20 Department. 
//  Tel: 03-5770066#8043
//  E-Mail: MicroC-8bit@nuvoton.com
//***********************************************************************************************************

#include <stdio.h>
#include "N79E825.h"
#include "Typedef.h"
#include "Define.h"
#include "Common.h"
#include "Delay.h"
#include "NVM.h"
#include "Absacc.h"


UINT8 mNVMBuffer[64] = 
        {
            0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
            0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
            0x55,0xAA,0x55,0xAA,0xFF,0x00,0xFF,0x00,0x55,0xAA,0x55,0xAA,0x00,0xFF,0x00,0xFF,
            0x00,0xFF,0x00,0xFF,0x55,0xAA,0x55,0xAA,0xFF,0x00,0xFF,0x00,0xAA,0x55,0xAA,0x55,
        };
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Erase(UINT16 u16Page_No)
{
    BIT bitTmp;

    /* MCU must clear all interrupt */
    bitTmp = EA;
    EA = 0;
    
    switch(u16Page_No)
    {
    case 0: 
        NVMADDR = 0x00;   // NVM PAGE0
        break;    
    case 1:       
        NVMADDR = 0x40;   // NVM PAGE1
        break;    
    case 2:       
        NVMADDR = 0x80;   // NVM PAGE2
        break;    
    case 3:       
        NVMADDR = 0xc0;   // NVM PAGE3
        break;
    default: 
        break;
    }
    /* Triger the NVM Page Erase */
    set_EER;              // EER = 1 do erase
    /* Restore the EA */
    EA = bitTmp;
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Write(UINT16 u16Page_No)
{
    UINT8 u8CNT, NVMTemp;
    BIT bitTmp;

    /* MCU must clear all interrupt */
    bitTmp = EA;
    EA = 0;

    u16Page_No = u16Page_No*64;

    for (u8CNT =0; u8CNT<64; u8CNT++)
    {
        /* Write NVM Page Address */
        NVMADDR = u16Page_No;
        /* Write NVM Page Data */
        NVMDAT = mNVMBuffer[u8CNT];
        /* Triger the NVM Page Write */
        set_EWR;
        /* Check ready, EWR=0 if finished */
        while(1)
        {
            NVMTemp = NVMCON;
            if((NVMTemp & 0x40) == 0)
            break;            
                
        }
        u16Page_No++;
    }
    EA = bitTmp;
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Write_Error(void)
{
    P1 = 0x78;
    printf("/nNVM Write Fail!");
    while(1);
}
//-----------------------------------------------------------------------------------------------------------
void NVM_Page_Read_Verify(UINT16 u16Page_No)
{
    UINT8 u8CNT,u8Tmp;

    u16Page_No = u16Page_No*64;
    for (u8CNT =0; u8CNT<64; u8CNT++)
    {
        /* Read NVM Page Data */
        u8Tmp = CBYTE[0xfc00+u16Page_No];
        if(u8Tmp != mNVMBuffer[u8CNT])
        {
            NVM_Write_Error();
        }    
        u16Page_No ++;
    }
}
//-----------------------------------------------------------------------------------------------------------
void main(void)
{
    InitialUART0_Timer1(9600);                  /* 9600 Baud Rate @ 11.0592MHz */

    printf ("\n*===================================================================");
    printf ("\n*  Name: N79E825 Series NVM Demo Code.");
    printf ("\n*  Pass show 0x00 to P1.");
    printf ("\n*  Fail show 0x78 to P1.");
    printf ("\n*===================================================================\n");

    NVM_Page_Erase(1);
    NVM_Page_Write(1);
    NVM_Page_Read_Verify(1);
    	

    printf("/nNVM Demo Pass");
    while(1)
	{
    P06 = 0;
    Delay1ms(1000);
	P06 = 1;
	Delay1ms(1000);
	}   
}
//-----------------------------------------------------------------------------------------------------------